package com.jiayue.insu.inclientqn.service.client;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.jiayue.insu.inclientqn.constant.CommonStant;
import com.jiayue.insu.inclientqn.entity.BackForeUsForecastData;
import com.jiayue.insu.inclientqn.entity.BackStatData;
import com.jiayue.insu.inclientqn.entity.Record;
import com.jiayue.insu.inclientqn.entity.Station;
import com.jiayue.insu.inclientqn.inenum.QNHLEnum;
import com.jiayue.insu.inclientqn.inenum.StatusEnum;
import com.jiayue.insu.inclientqn.model.RequestVo;
import com.jiayue.insu.inclientqn.model.ResponseVo;
import com.jiayue.insu.inclientqn.permisson.com.ComPermisson;
import com.jiayue.insu.inclientqn.service.BackForeUsForecastDataService;
import com.jiayue.insu.inclientqn.service.BackStatDataService;
import com.jiayue.insu.inclientqn.service.RecordService;
import com.jiayue.insu.inclientqn.service.StationService;
import com.jiayue.insu.inclientqn.service.client.BackDataGenService;
import com.jiayue.insu.inclientqn.util.RequestUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;


/**
 * @description: 回传数据请求
 * @author: yuanhao
 * @createDate: 2022/6/17
 * @version: 1.0
 */
@Service
@Slf4j
@RequiredArgsConstructor
@RefreshScope
public class RequestDataService {


    private final BackDataGenService backDataGenService;

    private final StationService stationService;

    private final ComPermisson comPermisson;

    private final RecordService recordService;

    private final BackStatDataService backStatDataService;

    private final BackForeUsForecastDataService backForeUsForecastDataService;

    public void requestBackStatData(QNHLEnum qnhlEnum, long signTime) {

        LocalDateTime localDateTime = LocalDateTimeUtil.of(signTime);

        Record record = new Record();
        record.setType(CommonStant.RECORD_TYPE_BACK_DATA);
        record.setTime(localDateTime);
        record.setCreateTime(LocalDateTime.now());
        record.setContent(qnhlEnum.getCode());


        Station station = stationService.findThis();


        List<BackStatData> backStatDatas = backStatDataService.findBySignTime(localDateTime);

        RequestVo requestVo = null;
        try {

            switch (qnhlEnum.getCode()) {
                case "RealPower":
                    requestVo = backDataGenService.genStateRealPowerData(station.getSignCode(), signTime, backStatDatas);
                    break;
                case "RealWeather":
                    requestVo = backDataGenService.genStateRealWeatherData(station.getSignCode(), station.getType(), signTime, backStatDatas);
                    break;
                case "ForecastPowerReport":
                    requestVo = backDataGenService.genStateForecastPowerReport(station.getSignCode(), signTime, backStatDatas);
                    break;
                case "UltraShortTermForecastReport":
                    requestVo = backDataGenService.genStateUltraShortTermForecastReport(station.getSignCode(), signTime, backStatDatas);
                    break;
            }

        } catch (Exception e) {
            e.printStackTrace();
            record.setState(StatusEnum.BACK_DATA_GEN_FAIL.getSign());
            record.setStateContent(StatusEnum.BACK_DATA_GEN_FAIL.getMsg());
        }
        //校验请求数据是否为null  和  token是否有效
        if (requestVo != null && comPermisson.isToken(station)) {
            JSONObject json = JSONUtil.parseObj(requestVo, true);
            String body = json.toString();
            SortedMap<Object, Object> sortedMap = new TreeMap<>();
            sortedMap.put("transferType", requestVo.getTransferType());
            sortedMap.put("date", requestVo.getDate());
            sortedMap.put("dataType", requestVo.getDataType());
            sortedMap.put("fileName", requestVo.getFileName());
            sortedMap.put("data", requestVo.getData());
            String s = JSONUtil.toJsonStr(sortedMap);
            String response = RequestUtils.post(station.getBackurl(), body, station.getComKey());

            if (StrUtil.isNotEmpty(response)) {

                boolean isJson = JSONUtil.isJsonObj(response);

                if (isJson) {

                    ResponseVo responseVo = JSONUtil.toBean(response, ResponseVo.class);
                    String code = responseVo.getRetCode();

                    if (code.equals(QNHLEnum.REQUEST_SUCCESS.getCode())) {

                        record.setState(StatusEnum.SUCCESS.getSign());
                        record.setStateContent(responseVo.getRetMsg());

                        BackStatData backStatData = backStatDatas.get(0);

                        switch (qnhlEnum.getCode()) {

                            case "RealPower":
                                backStatData.setBackTimeRealPower(LocalDateTime.now());
                                break;
                            case "RealWeather":
                                backStatData.setBackTimeRealWeather(LocalDateTime.now());
                                break;
                            case "ForecastPowerReport":
                                backStatData.setBackTimeForecastReport(LocalDateTime.now());
                                break;
                            case "UltraShortTermForecastReport":
                                backStatData.setBackTimeUsForecastReport(LocalDateTime.now());
                                break;
                        }

                        try {
                            backStatDataService.save(backStatData);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }

                    } else {
                        record.setState(code);
                        record.setStateContent(responseVo.getRetMsg());
                    }
                } else {
                    record.setState(StatusEnum.CONNECT_RESPONSE_FORMAT_ERROR.getSign());
                    record.setStateContent(StatusEnum.CONNECT_RESPONSE_FORMAT_ERROR.getMsg());
                }
            } else {

                record.setState(StatusEnum.CONNECT_RESPONSE_CONTENT_NULL.getSign());
                record.setStateContent(StatusEnum.CONNECT_RESPONSE_CONTENT_NULL.getMsg());
            }
        } else {
            record.setState(StatusEnum.BACK_DATA_GEN_FAIL.getSign());
            record.setStateContent("token失效或请求体为null");
        }


        recordService.save(record);
    }





    public void requestBackForeData(QNHLEnum qnhlEnum, long signTime) {

        LocalDateTime localDateTime = LocalDateTimeUtil.of(signTime);

        Record record = new Record();
        record.setType(CommonStant.RECORD_TYPE_BACK_DATA);
        record.setTime(localDateTime);
        record.setCreateTime(LocalDateTime.now());
        record.setContent(qnhlEnum.getCode());


        Station station = stationService.findThis();

        List<BackForeUsForecastData> list = backForeUsForecastDataService.findBySignTime(localDateTime);

        RequestVo requestVo = null;
        try {
            requestVo = backDataGenService.genForeUltraShortTermForecast(station.getSignCode(), signTime, list);

        }catch (Exception e){
            e.printStackTrace();
        }
    }


}
